@@ -61,17 +61,25 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Map;
+import java.util.Optional;
 
 import static org.apache.flink.streaming.connectors.elasticsearch.table.TestContext.context;
 import static org.apache.flink.table.api.Expressions.row;
 import static org.assertj.core.api.Assertions.assertThat;
+import static org.testcontainers.containers.wait.strategy.Wait.forHttp;
 
 /** IT tests for {@link Elasticsearch6DynamicSink}. */
 public class Elasticsearch6DynamicSinkITCase extends TestLogger {
 
     @ClassRule
     public static ElasticsearchContainer elasticsearchContainer =
-            new ElasticsearchContainer(DockerImageName.parse(DockerImageVersions.ELASTICSEARCH_6));
+            new ElasticsearchContainer(DockerImageName.parse(DockerImageVersions.ELASTICSEARCH_6))
+                    .waitingFor(
+                            forHttp("/")
+                                    .withMethod("HEAD")
+                                    .forStatusCode(200)
+                                    .forPort(9200)
+                                    .withStartupTimeout(Duration.ofMinutes(2)));
 
     @SuppressWarnings("deprecation")
     protected final RestHighLevelClient getClient() {
@@ -106,12 +114,11 @@ public class Elasticsearch6DynamicSinkITCase extends TestLogger {
 
         String index = "writing-documents";
         String myType = "MyType";
-        Elasticsearch6DynamicSinkFactory sinkFactory = new Elasticsearch6DynamicSinkFactory();
+        Elasticsearch6DynamicTableFactory factory = new Elasticsearch6DynamicTableFactory();
 
         SinkFunctionProvider sinkRuntimeProvider =
                 (SinkFunctionProvider)
-                        sinkFactory
-                                .createDynamicTableSink(
+                        factory.createDynamicTableSink(
                                         context()
                                                 .withSchema(schema)
                                                 .withOption(
@@ -230,11 +237,8 @@ public class Elasticsearch6DynamicSinkITCase extends TestLogger {
 
     @Test
     public void testWritingDocumentsNoPrimaryKey() throws Exception {
-        EnvironmentSettings settings = EnvironmentSettings.inStreamingMode();
-        settings.getConfiguration().setString("restart-strategy", "fixed-delay");
-        settings.getConfiguration().setInteger("restart-strategy.fixed-delay.attempts", 3);
-        // default fixed delay is 1 seconds
-        TableEnvironment tableEnvironment = TableEnvironment.create(settings);
+        TableEnvironment tableEnvironment =
+                TableEnvironment.create(EnvironmentSettings.inStreamingMode());
 
         String index = "no-primary-key";
         String myType = "MyType";
@@ -398,5 +402,9 @@ public class Elasticsearch6DynamicSinkITCase extends TestLogger {
                 DataType consumedDataType) {
             return null;
         }
+
+        public Optional<int[][]> getTargetColumns() {
+            return Optional.empty();
+        }
     }
 }
